#include <iostream>
#include <vector>
#include <set>
#include <list>
#include <algorithm>
#include <fstream>
#include <map>
#include <Windows.h>

using namespace std;

#define FINAL 1

const int kTimeLimit = 800;

struct Item{
	Item() { group = -1; }
	int color;
	int group;
	int x,y;

};

// A group is a collection of neighbour items of the same colour
struct Group{
	Group(int color,Item* item) : color(color),firstItem(item) { used = false; possible = false;size = 1; }
	vector<Group*> groups;
	set<Group*>    groupsSet;

	int color;
	bool used;
	bool possible;
	int size;

	Item* firstItem;

	std::list<Group*> addedState;
	std::list<Group*> changedState;
};

Item board[350][350];
// Groups sorted by size
vector<Group*> groups;
// Groups sorted by colour
vector<Group*> colors[500];
// Size of board,colours
int N,C;

// Solution holds the temporary solution,completeSolution holds the smallest solution found for the time allocated
vector<Group*> solution,completeSolution;
int start;
int maxDepth;

std::vector<Group*>::iterator firstNotUsed;
vector<Group*>* groupsWithThisColor;

bool comp(const Group* g1,const Group* g2);
void finish();

// If flat is true this function walks through all permutations up to the maxDepth level.
// If flat is false it will go to the max level nad return the first solution.
// The algorithm behind is greedy; the groups are sorted by decreasing size and connected accordingly.
// Look in my main to see how it is used and why.

template<bool flat>
inline void solve(Group* g,int depth = 1)
{
	if ( GetTickCount() - start > kTimeLimit ){
		finish();
	}

	if ( completeSolution.size() < (solution.size()+1) &&  completeSolution.size() != 0 ){
		return;
	}

	// Store the state in these lists, so we can restore it later
	if ( flat ){
		g->addedState.clear();
		g->changedState.clear();
	}

	// Flood :
	// IF there are any groups colored like the current one in the possibilities we can directly add them
	groupsWithThisColor = &colors[g->color];
	for(auto i = groupsWithThisColor->begin(); i != groupsWithThisColor->end(); i++){
		if ( !(*i)->used && (*i)->possible ){
			if ( flat ){
				g->addedState.insert(g->addedState.end(),*i);
			}

			(*i)->used = true;

			// We use this group - add all it's children as possibilties
			for(auto j = (*i)->groups.begin(); j != (*i)->groups.end(); j++){
				if (! (*j)->possible ){
					(*j)->possible = true;
					if ( flat ){
						g->changedState.insert(g->changedState.end(),*j);
					}
				}
			}
		}
	}

	// We use this group - add all it's children as possibilties
	for(auto i = g->groups.begin(); i != g->groups.end(); i++){
		if ( !(*i)->possible ){
			(*i)->possible = true;
			if ( flat ){
				g->changedState.insert(g->changedState.end(),*i);
			}
		}
	}

	solution.push_back(g);
	g->used = true;

	// Find the first possible group that is not used;
	// Go down recursively
	bool morePossibleGroups = false;
	auto oldNotUsed = firstNotUsed;

	for(auto i = oldNotUsed; i != groups.end(); i++){
		if ( !(*i)->used ){
			firstNotUsed = i;
			break;
		}
	}

	for(auto i = firstNotUsed; i != groups.end(); i++){
		if ( !(*i)->used ){

			if (  (*i)->possible ){
				morePossibleGroups = true;
				solve<flat>(*i,depth+1);
			
				if ( !flat ){
					goto next;
				}
				else{
					if ( depth > maxDepth ){
						goto next;
					}
				}
			}
		}
	}
	firstNotUsed = oldNotUsed;

next:
	if ( !morePossibleGroups ){
		completeSolution = solution;
	}


	// Resotre state
	if  ( flat ){
		for(auto i = g->addedState.begin(); i != g->addedState.end(); i++){
			(*i)->used = false;
		}
		for(auto i = g->changedState.begin(); i != g->changedState.end(); i++){
			(*i)->possible = false;
		}
		solution.pop_back();
		g->used = false;
	}

}

int main(int argc,char* argv[]){
	start = GetTickCount ();
	groups.reserve(350*350);
	solution.reserve(350*350);

#if FINAL
	ifstream fileIn("flood-it.in");
	ofstream fileOut("flood-it.out");
	cin.rdbuf(fileIn.rdbuf());
	cout.rdbuf(fileOut.rdbuf());
#endif

	cin >> N;
	cin >> C;

	int tt = GetTickCount();
	for(int i = 0; i < N;i++){
		for(int j = 0; j < N;j++){
			cin >> board[i][j].color;
			board[i][j].x = i;
			board[i][j].y = j;
		}
	}	

	// Find and create groups : Item that are neigbours and have the same colour.
	// Groups are very useful, beacuse we don't care how many or which items thety consist of - we only care of their surrounding groups
	// It doesn't matter what item in the grou we select as a starting point for our solution.
	// This simplifies things - in the task example there are just 13 groups, but there are 25 items
	for(int i = 0; i < N;i++){
		for(int j = 0; j < N;j++){
			if ( board[i][j].group == -1){
				board[i][j].group = groups.size();
				groups.push_back(new Group(board[i][j].color,&board[i][j]));
				colors[board[i][j].color].push_back(groups.back());
			}
			int a = i+1;
			int b = j+1;
			if ( a < N ){
				if ( board[a][j].color == board[i][j].color ){
					board[a][j].group = board[i][j].group;
					groups[board[a][j].group]->size++;
				}
			}
			if ( b < N ){
				if ( board[i][b].color == board[i][j].color ){
					board[i][b].group = board[i][j].group;
					groups[board[i][b].group]->size++;
				}
			}
		}
	}

	// Find neighbour (possible) groups for every group
	for(int i = 0; i < N;i++){
		for(int j = 0; j < N;j++){
			int a = i+1;
			int b = j+1;
			if ( a < N ){
				if ( board[a][j].group != board[i][j].group ){
					groups[board[a][j].group]->groupsSet.insert( groups[board[i][j].group] );
					groups[board[i][j].group]->groupsSet.insert( groups[board[a][j].group] );
				}
			}
			if ( b < N ){
				if ( board[i][b].group != board[i][j].group ){
					groups[board[i][b].group]->groupsSet.insert( groups[board[i][j].group] );
					groups[board[i][j].group]->groupsSet.insert( groups[board[i][b].group] );
				}
			}
		}
	}
	
	for(int i = 0; i < groups.size();i++){
		groups[i]->groups.assign(groups[i]->groupsSet.begin(),groups[i]->groupsSet.end());
	}

	// Sort groups by decreasing size
	std::sort(groups.begin(),groups.end(),comp);

	//cout << groups.size() << endl;
	// Now quickly solve the problem by goiing through groups in a greedy fashion. 
	// Note that I pass false to the template argument - this finds the greedy solution just for the group I'm passing in
	// and is very fast. I do this first, because for big fields there may not be enough time to attempt anything else.
	for(int i = 0; i < (int)groups.size();i++){
		firstNotUsed = groups.begin();
		solve<false>(groups[i]);

		// We don't use state restore when I pass false to solve (faster), so I do it here.
		solution.clear();
		for(int j = 0; j < (int)groups.size();j++){
			groups[j]->used = groups[j]->possible = false;
		}
	}

	
	// Exsaust all possibilities (Very slow, but it doesn't matter :) 
	// We already found our solution in the loop above - we are just trying to improve it here,
	// since there is more time
	maxDepth = 1;
	while(true){
		for(int i = 0; i < (int)groups.size();i++){
			firstNotUsed = groups.begin();
			solve<true>(groups[i]);
		}
		maxDepth++;
	}

	finish();

	return 0;
}

bool comp(const Group* g1,const Group* g2)
{
	return g1->groups.size() > g2->groups.size();
}

// Rude, I know :)
void finish()
{
	cout << completeSolution.size() - 1 << endl;
	cout << completeSolution.front()->firstItem->x << " " << completeSolution.front()->firstItem->y << endl;
	auto i = completeSolution.begin();
	i++;
	for(; i != completeSolution.end(); i++){
		cout << (*i)->color << endl;
	}


	exit(0);
}
